@@ -1,9 +1,10 @@
 //-< JLINT.CC >------------------------------------------------------+--------+
-// Jlint                      Version 2.2        (c) 1998  GARRET    |     ?  |
+// Jlint                      Version 3.0        (c) 1998  GARRET    |     ?  |
 // (Java Lint)                                                       |   /\|  |
 //                                                                   |  /  \  |
-//                          Created:     28-Mar-98    K.A. Knizhnik  | / [] \ |
-//                          Last update: 05-Jun-01    Cyrille Artho  | GARRET |
+//   Created:                    28-Mar-98    K.A. Knizhnik          | / [] \ |
+//   Version 2.X:   Last update: 05-Jun-01    Cyrille Artho          | GARRET |
+//   Version 3.X:   Last update: 20-Aug-03    Raphael Ackermann      |        |
 //-------------------------------------------------------------------+--------+
 // Java verifier 
 //-------------------------------------------------------------------+--------+
@@ -22,6 +23,12 @@
 
 #include "jlint.hh"
 
+extern "C"
+{
+#include <zlib.h>        // should I use <zlib.h> or "zlib.h" ??
+  // #include "zlib.h"
+};
+
 bool   verbose = false;
 int   max_shown_paths = 4;
 
@@ -35,12 +42,13 @@ FILE* history;
 string_pool stringPool;
 field_desc* is_const;
 
-message_descriptor msg_table[] = {
+message_descriptor msg_table[] = 
+  {
 #define MSG(category, code, position_dependent, format) \
-{cat_##category, MSG_LOCATION_PREFIX##format, #code, position_dependent, true},
+   {cat_##category, MSG_LOCATION_PREFIX format, #code, position_dependent, true},
 #include "jlint.msg"
-  {cat_all}
-};
+    {cat_all}
+  };
 
 unsigned int string_hash_function(byte* p) { 
   unsigned int h = 0, g;
@@ -503,10 +511,45 @@ bool parse_class_file(byte* fp)
                sizeof(local_context*)*(code_length+1));
 
         int exception_table_length = unpack2(fp); fp += 2;
+
+	/* add new entry for each distinct "byte code adress of handle".
+	**
+	** if an exception handler at byte code "pos" handles exception of
+	** more than one byte code range, call
+	** "new ctx_entry_point(&method->context[pos]);" only once! Because
+	** otherwise the stack gets out of control. 
+	**
+	** in the following example there are two different handle adresses
+	** 16 and 25. and for each of them 
+	** "new ctx_entry_point(&method->context[handler_pc]);" is called 
+	** exactly once. Therefore the program calls :
+	** new ctx_entry_point(&method->context[16]);
+	** new ctx_entry_point(&method->context[25]);
+	**********************************************************************
+	** Example Exception Table:                                         **
+	** -------------------------------------                            **
+	**                                                                  **
+	**                      byte code adress                            **
+	** from         to        of handle                                 **
+	**  2           10           16                                     **
+	** 12           14           16                                     **
+	** 20           23           25                                     **
+	**********************************************************************
+	**
+	** it is expected that the byte code adresses of the handles are 
+	** ordered. If this would not be the case, a simple comparison of
+	** handler_pc and old_handler_pc would not be sufficient!
+	*/
+
+	int old_handler_pc = -1;
+
         while (--exception_table_length >= 0) { 
           int handler_pc = unpack2(fp+4);
-          new ctx_entry_point(&method->context[handler_pc]); 
-          fp += 8;
+	  if ( handler_pc != old_handler_pc) {
+	    new ctx_entry_point(&method->context[handler_pc]); 
+	  }
+	  fp += 8;
+	  old_handler_pc = handler_pc;
         }
 
         int method_attr_count = unpack2(fp); fp += 2;
@@ -713,6 +756,8 @@ void proceed_file(char* file_name, bool recursive = false)
     byte* dp = directory;
 
     while (--count >= 0) {
+      int compression_method = unpack2_le(&dp[4+C_COMPRESSION_METHOD]);
+      int compressed_size = unpack4_le(&dp[4+C_COMPRESSED_SIZE]);
       int uncompressed_size = unpack4_le(&dp[4+C_UNCOMPRESSED_SIZE]);
       int filename_length = unpack2_le(&dp[4+C_FILENAME_LENGTH]);
       int cextra_length = unpack2_le(&dp[4+C_EXTRA_FIELD_LENGTH]);
@@ -740,25 +785,86 @@ void proceed_file(char* file_name, bool recursive = false)
 
       int filename_offset = CREC_SIZE+4;
 
-      if (uncompressed_size != 0) { 
-        byte* buffer = new byte[uncompressed_size];
-        if (fseek(f, file_start, SEEK_SET) != 0 
-            || fread(buffer, uncompressed_size, 1, f) != 1) 
-          { 
-            fprintf(stderr, "Failed to read ZIP file '%s'\n", 
-                    file_name);
-            return;
-          } else { 
-            if (verbose) { 
-              fprintf(stderr, "Verify file '%.*s'\n", 
-                      filename_length, dp + filename_offset); 
-            }
-            if (!parse_class_file(buffer)) { 
-              fprintf(stderr, "File '%.*s' isn't correct Java class "
-                      "file\n", filename_length, dp+filename_offset);
-            }
+      if (compression_method == C_UNCOMPRESSED)
+	{
+          if (uncompressed_size != 0) { 
+            byte* buffer = new byte[uncompressed_size];
+            if (fseek(f, file_start, SEEK_SET) != 0 
+                || fread(buffer, uncompressed_size, 1, f) != 1) 
+              { 
+                fprintf(stderr, "Failed to read ZIP file '%s'\n", 
+                        file_name);
+                return;
+              } else { 
+                if (verbose) { 
+                  fprintf(stderr, "Verify file '%.*s'\n", 
+                          filename_length, dp + filename_offset); 
+                }
+                if (!parse_class_file(buffer)) { 
+                  fprintf(stderr, "File '%.*s' isn't correct Java class "
+                          "file\n", filename_length, dp+filename_offset);
+                }
+              }
+            delete[] buffer;
+          }
+      }
+      else if (compression_method == C_DEFLATE) {
+
+          if ((uncompressed_size != 0) && (compressed_size != 0)) { 
+            byte* uncbuffer = new byte[uncompressed_size];
+            byte* cbuffer = new byte[compressed_size];
+            if (fseek(f, file_start, SEEK_SET) != 0 
+                || fread(cbuffer, compressed_size, 1, f) != 1) 
+              { 
+                fprintf(stderr, "Failed to read ZIP file '%s'\n", 
+                        file_name);
+                return;
+              } else { 
+                z_stream c_stream;
+
+                c_stream.zalloc = (alloc_func) 0;
+                c_stream.zfree = (free_func)0;
+                c_stream.opaque = (voidpf)0;
+
+                int err = inflateInit2(&c_stream, -MAX_WBITS);
+                if (err != Z_OK)
+                {
+                    fprintf(stderr, "Error %s in inflateInit while uncompressing %.*s.\n",
+                        c_stream.msg, filename_length, dp+filename_offset);
+                }
+                else
+                {
+                    c_stream.next_in = cbuffer;
+                    c_stream.next_out = uncbuffer;
+                    c_stream.avail_in = compressed_size;
+                    c_stream.avail_out = uncompressed_size;
+                    c_stream.total_in = 0;
+                    c_stream.total_out = 0;
+
+                    while ((err=inflate(&c_stream, Z_SYNC_FLUSH)) == Z_OK);
+                    if ((err != Z_STREAM_END) && (err != Z_OK))
+                    {
+                        fprintf(stderr, "Error %s in inflate while uncompressing %.*s.\n",
+                            c_stream.msg, filename_length, dp+filename_offset);
+                    }
+                    else
+                    {
+                        inflateEnd(&c_stream);
+
+                        if (verbose) { 
+                            fprintf(stderr, "Verify file '%.*s'\n", 
+                            filename_length, dp + filename_offset); 
+                        }
+                        if (!parse_class_file(uncbuffer)) { 
+                            fprintf(stderr, "File '%.*s' isn't correct Java class "
+                                "file\n", filename_length, dp+filename_offset);
+                        }
+                    }
+                }
+              }
+            delete[] cbuffer;
+            delete[] uncbuffer;
           }
-        delete[] buffer;
       }
       dp += filename_offset + filename_length + cextra_length; 
     }
@@ -796,33 +902,33 @@ msg_select_category_option msg_category_option[] = {
   {cat_race_condition, "race_condition", 
      "possible race condition detection"},
   {cat_wait_nosync, "wait_nosync", 
-     "wait or notify is called from non-synchroize method"},
+     "wait or notify is called from non-synchronized method"},
   {cat_inheritance, "inheritance",
      "detect problems with class/interface inheritance"},
   {cat_super_finalize, "super_finalize", 
      "super.finalize() is not called from finalize method"},
   {cat_not_overridden, "not_overridden",
-     "methods with the same names and diffent profiles"},
+     "methods with the same names and different profiles"},
   {cat_field_redefined, "field_redefined",
-     "instance or static variable is redifined in derived class"},
+     "instance or static variable is redefined in derived class"},
   {cat_shadow_local, "shadow_local",
-     "local variable shadow one in exterior scope"},
+     "local variable shadows one in exterior scope"},
   {cat_data_flow, "data_flow",
      "perform data flow analysis to detect suspicious operations"},
   {cat_null_reference, "null_reference",
      "detect access to variables with possible NULL value"},
   {cat_zero_operand, "zero_operand",
-     "one of operands of binary integer operation is zero"},
+     "one of the operands of a binary integer operation is zero"},
   {cat_zero_result, "zero_result",
      "result of operation is always zero"},
   {cat_redundant, "redundant",
-     "operations always produce the same result"},
+     "operation always produces the same result"},
   {cat_overflow, "overflow", 
      "detect incorrect handling of operation overflow"},
   {cat_incomp_case, "incomp_case",
      "switch case label can't be produced by switch expression"},
   {cat_short_char_cmp, "short_char_cmp",
-     "expresion of char type is compared with one of short type"},
+     "expression of char type is compared with one of short type"},
   {cat_string_cmp, "string_cmp", 
      "two strings are compared as object references"},
   {cat_weak_cmp, "weak_cmp", 
